@@ -44,18 +44,6 @@ DISubprogram *getDISubprogram(const MDNode *Scope);
 bool StripDebugInfo(Module &M);
 bool stripDebugInfo(Function &F);
 
-/// Downgrade the debug info in a module to contain only line table information.
-///
-/// In order to convert debug info to what -gline-tables-only would have
-/// created, this does the following:
-///   1) Delete all debug intrinsics.
-///   2) Delete all non-CU named metadata debug info nodes.
-///   3) Create new DebugLocs for each instruction.
-///   4) Create a new CU debug info, and similarly for every metadata node
-///      that's reachable from the CU debug info.
-///   All debug type metadata nodes are unreachable and garbage collected.
-bool stripNonLineTableDebugInfo(Module &M);
-
 /// \brief Return Debug Info Metadata Version by checking module flags.
 unsigned getDebugMetadataVersionFromModule(const Module &M);
 
@@ -89,7 +77,7 @@ private:
   void processSubprogram(DISubprogram *SP);
   void processScope(DIScope *Scope);
   bool addCompileUnit(DICompileUnit *CU);
-  bool addGlobalVariable(DIGlobalVariableExpression *DIG);
+  bool addGlobalVariable(DIGlobalVariable *DIG);
   bool addSubprogram(DISubprogram *SP);
   bool addType(DIType *DT);
   bool addScope(DIScope *Scope);
@@ -98,8 +86,8 @@ public:
   typedef SmallVectorImpl<DICompileUnit *>::const_iterator
       compile_unit_iterator;
   typedef SmallVectorImpl<DISubprogram *>::const_iterator subprogram_iterator;
-  typedef SmallVectorImpl<DIGlobalVariableExpression *>::const_iterator
-      global_variable_expression_iterator;
+  typedef SmallVectorImpl<DIGlobalVariable *>::const_iterator
+      global_variable_iterator;
   typedef SmallVectorImpl<DIType *>::const_iterator type_iterator;
   typedef SmallVectorImpl<DIScope *>::const_iterator scope_iterator;
 
@@ -111,7 +99,7 @@ public:
     return make_range(SPs.begin(), SPs.end());
   }
 
-  iterator_range<global_variable_expression_iterator> global_variables() const {
+  iterator_range<global_variable_iterator> global_variables() const {
     return make_range(GVs.begin(), GVs.end());
   }
 
@@ -132,7 +120,7 @@ public:
 private:
   SmallVector<DICompileUnit *, 8> CUs;
   SmallVector<DISubprogram *, 8> SPs;
-  SmallVector<DIGlobalVariableExpression *, 8> GVs;
+  SmallVector<DIGlobalVariable *, 8> GVs;
   SmallVector<DIType *, 8> TYs;
   SmallVector<DIScope *, 8> Scopes;
   SmallPtrSet<const MDNode *, 32> NodesSeen;
