@@ -1,31 +1,32 @@
-//===- llvm/MC/MCELFObjectWriter.h - ELF Object Writer ----------*- C++ -*-===//
+//===-- llvm/MC/MCELFObjectWriter.h - ELF Object Writer ---------*- C++ -*-===//
 //
-// Part of the LLVM Project, under the Apache License v2.0 with LLVM Exceptions.
-// See https://llvm.org/LICENSE.txt for license information.
-// SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
+//                     The LLVM Compiler Infrastructure
+//
+// This file is distributed under the University of Illinois Open Source
+// License. See LICENSE.TXT for details.
 //
 //===----------------------------------------------------------------------===//
 
 #ifndef LLVM_MC_MCELFOBJECTWRITER_H
 #define LLVM_MC_MCELFOBJECTWRITER_H
 
-#include "llvm/BinaryFormat/ELF.h"
-#include "llvm/MC/MCObjectWriter.h"
-#include "llvm/MC/MCSectionELF.h"
-#include "llvm/Support/Casting.h"
+#include "llvm/ADT/Triple.h"
+#include "llvm/MC/MCValue.h"
+#include "llvm/Support/DataTypes.h"
+#include "llvm/Support/ELF.h"
 #include "llvm/Support/raw_ostream.h"
-#include "llvm/TargetParser/Triple.h"
-#include <cstdint>
 #include <vector>
 
 namespace llvm {
-
 class MCAssembler;
 class MCContext;
 class MCFixup;
+class MCFragment;
+class MCObjectWriter;
 class MCSymbol;
 class MCSymbolELF;
 class MCValue;
+class raw_pwrite_stream;
 
 struct ELFRelocationEntry {
   uint64_t Offset; // Where is the relocation.
@@ -46,61 +47,53 @@ struct ELFRelocationEntry {
         << ", Addend=" << Addend << ", OriginalSymbol=" << OriginalSymbol
         << ", OriginalAddend=" << OriginalAddend;
   }
-
-  LLVM_DUMP_METHOD void dump() const { print(errs()); }
+  void dump() const { print(errs()); }
 };
 
-class MCELFObjectTargetWriter : public MCObjectTargetWriter {
+class MCELFObjectTargetWriter {
   const uint8_t OSABI;
-  const uint8_t ABIVersion;
   const uint16_t EMachine;
   const unsigned HasRelocationAddend : 1;
   const unsigned Is64Bit : 1;
+  const unsigned IsN64 : 1;
 
 protected:
-  MCELFObjectTargetWriter(bool Is64Bit_, uint8_t OSABI_, uint16_t EMachine_,
-                          bool HasRelocationAddend_, uint8_t ABIVersion_ = 0);
+
+  MCELFObjectTargetWriter(bool Is64Bit_, uint8_t OSABI_,
+                          uint16_t EMachine_,  bool HasRelocationAddend,
+                          bool IsN64=false);
 
 public:
-  virtual ~MCELFObjectTargetWriter() = default;
-
-  Triple::ObjectFormatType getFormat() const override { return Triple::ELF; }
-  static bool classof(const MCObjectTargetWriter *W) {
-    return W->getFormat() == Triple::ELF;
-  }
-
   static uint8_t getOSABI(Triple::OSType OSType) {
     switch (OSType) {
-      case Triple::HermitCore:
-        return ELF::ELFOSABI_STANDALONE;
+      case Triple::CloudABI:
+        return ELF::ELFOSABI_CLOUDABI;
       case Triple::PS4:
       case Triple::FreeBSD:
         return ELF::ELFOSABI_FREEBSD;
-      case Triple::Solaris:
-        return ELF::ELFOSABI_SOLARIS;
       default:
         return ELF::ELFOSABI_NONE;
     }
   }
 
+  virtual ~MCELFObjectTargetWriter() {}
+
   virtual unsigned getRelocType(MCContext &Ctx, const MCValue &Target,
                                 const MCFixup &Fixup, bool IsPCRel) const = 0;
 
-  virtual bool needsRelocateWithSymbol(const MCValue &Val, const MCSymbol &Sym,
+  virtual bool needsRelocateWithSymbol(const MCSymbol &Sym,
                                        unsigned Type) const;
 
   virtual void sortRelocs(const MCAssembler &Asm,
                           std::vector<ELFRelocationEntry> &Relocs);
 
-  virtual void addTargetSectionFlags(MCContext &Ctx, MCSectionELF &Sec);
-
   /// \name Accessors
   /// @{
   uint8_t getOSABI() const { return OSABI; }
-  uint8_t getABIVersion() const { return ABIVersion; }
   uint16_t getEMachine() const { return EMachine; }
   bool hasRelocationAddend() const { return HasRelocationAddend; }
   bool is64Bit() const { return Is64Bit; }
+  bool isN64() const { return IsN64; }
   /// @}
 
   // Instead of changing everyone's API we pack the N64 Type fields
@@ -129,38 +122,28 @@ public:
   }
 
   // N64 relocation type setting
-  static unsigned setRTypes(unsigned Value1, unsigned Value2, unsigned Value3) {
-    return ((Value1 & 0xff) << R_TYPE_SHIFT) |
-           ((Value2 & 0xff) << R_TYPE2_SHIFT) |
-           ((Value3 & 0xff) << R_TYPE3_SHIFT);
+  unsigned setRType(unsigned Value, unsigned Type) const {
+    return ((Type & R_TYPE_MASK) | ((Value & 0xff) << R_TYPE_SHIFT));
+  }
+  unsigned setRType2(unsigned Value, unsigned Type) const {
+    return (Type & R_TYPE2_MASK) | ((Value & 0xff) << R_TYPE2_SHIFT);
+  }
+  unsigned setRType3(unsigned Value, unsigned Type) const {
+    return (Type & R_TYPE3_MASK) | ((Value & 0xff) << R_TYPE3_SHIFT);
   }
   unsigned setRSsym(unsigned Value, unsigned Type) const {
     return (Type & R_SSYM_MASK) | ((Value & 0xff) << R_SSYM_SHIFT);
   }
-
-  // On AArch64, return a new section to be added to the ELF object that
-  // contains relocations used to describe every symbol that should have memory
-  // tags applied. Returns nullptr if no such section is necessary (i.e. there's
-  // no tagged globals).
-  virtual MCSectionELF *getMemtagRelocsSection(MCContext &Ctx) const {
-    return nullptr;
-  }
 };
 
-/// Construct a new ELF writer instance.
+/// \brief Construct a new ELF writer instance.
 ///
 /// \param MOTW - The target specific ELF writer subclass.
 /// \param OS - The stream to write to.
 /// \returns The constructed object writer.
-std::unique_ptr<MCObjectWriter>
-createELFObjectWriter(std::unique_ptr<MCELFObjectTargetWriter> MOTW,
-                      raw_pwrite_stream &OS, bool IsLittleEndian);
+MCObjectWriter *createELFObjectWriter(MCELFObjectTargetWriter *MOTW,
+                                      raw_pwrite_stream &OS,
+                                      bool IsLittleEndian);
+} // End llvm namespace
 
-std::unique_ptr<MCObjectWriter>
-createELFDwoObjectWriter(std::unique_ptr<MCELFObjectTargetWriter> MOTW,
-                         raw_pwrite_stream &OS, raw_pwrite_stream &DwoOS,
-                         bool IsLittleEndian);
-
-} // end namespace llvm
-
-#endif // LLVM_MC_MCELFOBJECTWRITER_H
+#endif
